@@ -22,6 +22,17 @@
 
    By tege@sics.se, Torbjorn Granlund, advised by rms, Richard Stallman. */
 
+#ifdef HAVE_CONFIG_H
+#if defined (CONFIG_BROKETS)
+/* We use <config.h> instead of "config.h" so that a compilation
+   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
+   (which it would do because it found this file in $srcdir).  */
+#include <config.h>
+#else
+#include "config.h"
+#endif
+#endif
+
 #include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
@@ -74,15 +85,37 @@ static int newlines2 = 0;
 static int exit_stat = 0;
 
 static void
-usage ()
+usage (status)
+     int status;
 {
-  fprintf (stderr, "\
-Usage: %s [-benstuvAET] [--number] [--number-nonblank] [--squeeze-blank]\n\
-       [--show-nonprinting] [--show-ends] [--show-tabs] [--show-all]\n\
-       [--help] [--version] [file...]\n",
-	   program_name);
-
-  exit (2);
+  if (status != 0)
+    fprintf (stderr, "Try `%s --help' for more information.\n",
+	     program_name);
+  else
+    {
+      printf ("\
+Usage: %s [OPTION] [FILE]...\n\
+",
+	      program_name);
+      printf ("\
+\n\
+  -b, --number-nonblank    number nonblank output lines\n\
+  -e                       equivalent to -vE\n\
+  -n, --number             number all output lines\n\
+  -s, --squeeze-blank      never more than one single blank line\n\
+  -t                       equivalent to -vT\n\
+  -u                       (ignored)\n\
+  -v, --show-nonprinting   use ^ and M- notation, save for LFD and TAB\n\
+  -A, --show-all           equivalent to -vET\n\
+  -E, --show-ends          display $ at end of each line\n\
+  -T, --show-tabs          display TAB characters as ^I\n\
+      --help               display this help and exit\n\
+      --version            output version information and exit\n\
+\n\
+With no FILE, or when FILE is -, read standard input.\n\
+");
+    }
+  exit (status);
 }
 
 
@@ -134,10 +167,10 @@ main (argc, argv)
   int options = 0;
 
   /* If non-zero, display usage information and exit.  */
-  static int flag_help;
+  static int show_help;
 
-  /* If non-zero, print the version on standard error.  */
-  static int flag_version;
+  /* If non-zero, print the version on standard output then exit.  */
+  static int show_version;
 
   static struct option const long_options[] =
   {
@@ -148,8 +181,8 @@ main (argc, argv)
     {"show-ends", no_argument, NULL, 'E'},
     {"show-tabs", no_argument, NULL, 'T'},
     {"show-all", no_argument, NULL, 'A'},
-    {"help", no_argument, &flag_help, 1},
-    {"version", no_argument, &flag_version, 1},
+    {"help", no_argument, &show_help, 1},
+    {"version", no_argument, &show_version, 1},
     {NULL, 0, NULL, 0}
   };
 
@@ -220,18 +253,18 @@ main (argc, argv)
 	  break;
 
 	default:
-	  usage ();
+	  usage (2);
 	}
     }
 
-  if (flag_version)
+  if (show_version)
     {
-      fprintf (stderr, "%s\n", version_string);
+      printf ("%s\n", version_string);
       exit (0);
     }
 
-  if (flag_help)
-    usage ();
+  if (show_help)
+    usage (0);
 
   output_desc = 1;
 
@@ -511,8 +544,12 @@ cat (inbuf, insize, outbuf, outsize, quote,
 		  && ioctl (input_desc, FIONREAD, &n_to_read) < 0)
 		{
 		  /* Ultrix returns EOPNOTSUPP on NFS;
-		     HP-UX returns ENOTTY on pipes. */
-		  if (errno == EOPNOTSUPP || errno == ENOTTY)
+		     HP-UX returns ENOTTY on pipes.
+		     SunOS returns EINVAL and
+		     More/BSD returns ENODEV on special files
+		     like /dev/null.  */
+		  if (errno == EOPNOTSUPP || errno == ENOTTY
+		      || errno == EINVAL || errno == ENODEV)
 		    use_fionread = 0;
 		  else
 		    {
