@@ -18,10 +18,24 @@
 /* Written by Stuart Kemp, cpsrk@groper.jcu.edu.au.
    Modified by David MacKenzie, djm@gnu.ai.mit.edu. */
 
+#ifdef HAVE_CONFIG_H
+#if defined (CONFIG_BROKETS)
+/* We use <config.h> instead of "config.h" so that a compilation
+   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
+   (which it would do because it found this file in $srcdir).  */
+#include <config.h>
+#else
+#include "config.h"
+#endif
+#endif
+
 #include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
 #include <signal.h>
+#ifdef HAVE_LIMITS_H
+#include <limits.h>
+#endif /* HAVE_LIMITS_H */
 #include "regex.h"
 #include "system.h"
 #include "version.h"
@@ -33,6 +47,14 @@ char *malloc ();
 char *realloc ();
 #endif
 
+#ifndef MAX
+#define MAX(a,b) (((a) > (b)) ? (a) : (b))
+#endif
+
+#ifndef INT_MAX
+#define INT_MAX (~(1 << (sizeof (int) * 8 - 1)))
+#endif /* INT_MAX */
+
 void error ();
 
 
@@ -122,6 +144,13 @@ struct buffer_record
 /* The name this program was run with. */
 char *program_name;
 
+/* Convert the number of 8-bit bytes of a binary representation to
+   the number of characters required to represent the same quantity
+   as an unsigned octal.  For example, a 32-bit (4-byte) quantity may
+   require a field width as wide as 11 characters.  */
+static const unsigned int bytes_to_octal_digits[] =
+{0, 3, 6, 8, 11, 14, 16, 19, 22, 25, 27, 30, 32, 35, 38, 41, 43};
+
 /* Input file descriptor. */
 static int input_desc = 0;
 
@@ -155,6 +184,9 @@ static char *filename_space = NULL;
 /* Prefix part of output file names. */
 static char *prefix = NULL;
 
+/* Suffix part of output file names. */
+static char *suffix = NULL;
+
 /* Number of digits to use in output file names. */
 static int digits = 2;
 
@@ -167,6 +199,9 @@ static unsigned bytes_written;
 /* Output file pointer. */
 static FILE *output_stream = NULL;
 
+/* Output file name. */
+static char *output_filename = NULL;
+
 /* Perhaps it would be cleaner to pass arg values instead of indexes. */
 static char **global_argv;
 
@@ -176,6 +211,9 @@ static boolean suppress_count;
 /* If TRUE, remove output files on error. */
 static boolean remove_files;
 
+/* If TRUE, remove all output files which have a zero length. */
+static boolean elide_empty_files;
+
 /* The compiled pattern arguments, which determine how to split
    the input file. */
 static struct control *controls;
@@ -184,20 +222,22 @@ static struct control *controls;
 static unsigned control_used;
 
 /* If non-zero, display usage information and exit.  */
-static int flag_help;
+static int show_help;
 
-/* If non-zero, print the version on standard error.  */
-static int flag_version;
+/* If non-zero, print the version on standard output then exit.  */
+static int show_version;
 
 static struct option const longopts[] =
 {
   {"digits", required_argument, NULL, 'n'},
-  {"quiet", no_argument, NULL, 's'},
+  {"quiet", no_argument, NULL, 'q'},
   {"silent", no_argument, NULL, 's'},
   {"keep-files", no_argument, NULL, 'k'},
+  {"elide-empty-files", no_argument, NULL, 'z'},
   {"prefix", required_argument, NULL, 'f'},
-  {"help", no_argument, &flag_help, 1},
-  {"version", no_argument, &flag_version, 1},
+  {"suffix", required_argument, NULL, 'b'},
+  {"help", no_argument, &show_help, 1},
+  {"version", no_argument, &show_version, 1},
   {NULL, 0, NULL, 0}
 };
 
@@ -591,7 +631,7 @@ get_first_line_in_buffer ()
   return head->first_available;
 }
 
-/* Return a pointer to the logical first line in the buffer and make the 
+/* Return a pointer to the logical first line in the buffer and make the
    next line the logical first line.
    Return NULL if there is no more input. */
 
@@ -832,7 +872,7 @@ process_regexp (p, repetition)
      int repetition;
 {
   struct cstring *line;		/* From input file. */
-  register unsigned line_len;	/* To make "$" in regexps work. */
+  unsigned line_len;	/* To make "$" in regexps work. */
   unsigned break_line;		/* First line number of next file. */
   boolean ignore = p->ignore;	/* If TRUE, skip this section. */
   int ret;
@@ -849,7 +889,19 @@ process_regexp (p, repetition)
 	{
 	  line = find_line (++current_line);
 	  if (line == NULL)
-	    regexp_error (p, repetition, ignore);
+	    {
+	      if (p->repeat == INT_MAX)
+		{
+		  if (!ignore)
+		    {
+		      dump_rest_of_file ();
+		      close_output_file ();
+		    }
+		  exit (0);
+		}
+	      else
+		regexp_error (p, repetition, ignore);
+	    }
 	  line_len = line->len;
 	  if (line->str[line_len - 1] == '\n')
 	    line_len--;
@@ -877,7 +929,19 @@ process_regexp (p, repetition)
 	{
 	  line = find_line (++current_line);
 	  if (line == NULL)
-	    regexp_error (p, repetition, ignore);
+	    {
+	      if (p->repeat == INT_MAX)
+		{
+		  if (!ignore)
+		    {
+		      dump_rest_of_file ();
+		      close_output_file ();
+		    }
+		  exit (0);
+		}
+	      else
+		regexp_error (p, repetition, ignore);
+	    }
 	  line_len = line->len;
 	  if (line->str[line_len - 1] == '\n')
 	    line_len--;
@@ -909,7 +973,7 @@ process_regexp (p, repetition)
 static void
 split_file ()
 {
-  register int i, j;
+  int i, j;
 
   for (i = 0; i < control_used; i++)
     {
@@ -936,7 +1000,11 @@ static char *
 make_filename (num)
      int num;
 {
-  sprintf (filename_space, "%s%0*d", prefix, digits, num);
+  strcpy (filename_space, prefix);
+  if (suffix)
+    sprintf (filename_space+strlen(prefix), suffix, num);
+  else
+    sprintf (filename_space+strlen(prefix), "%0*d", digits, num);
   return filename_space;
 }
 
@@ -945,13 +1013,11 @@ make_filename (num)
 static void
 create_output_file ()
 {
-  char *name;
-
-  name = make_filename (files_created);
-  output_stream = fopen (name, "w");
+  output_filename = make_filename (files_created);
+  output_stream = fopen (output_filename, "w");
   if (output_stream == NULL)
     {
-      error (0, errno, "%s", name);
+      error (0, errno, "%s", output_filename);
       cleanup ();
     }
   files_created++;
@@ -984,11 +1050,18 @@ close_output_file ()
     {
       if (fclose (output_stream) == EOF)
 	{
-	  error (0, errno, "write error");
+	  error (0, errno, "write error for `%s'", output_filename);
 	  cleanup ();
 	}
-      if (!suppress_count)
-	fprintf (stdout, "%d\n", bytes_written);
+      if (bytes_written == 0 && elide_empty_files)
+	{
+	  if (unlink (output_filename))
+	    error (0, errno, "%s", output_filename);
+	  files_created--;
+	}
+      else
+        if (!suppress_count)
+	  fprintf (stdout, "%d\n", bytes_written);
       output_stream = NULL;
     }
 }
@@ -1025,7 +1098,7 @@ static struct control *
 new_control_record ()
 {
   static unsigned control_allocated = 0; /* Total space allocated. */
-  register struct control *p;
+  struct control *p;
 
   if (control_allocated == 0)
     {
@@ -1056,8 +1129,8 @@ string_to_number (result, num)
      int *result;
      char *num;
 {
-  register char ch;
-  register int val = 0;
+  char ch;
+  int val = 0;
 
   if (*num == '\0')
     return FALSE;
@@ -1112,9 +1185,12 @@ parse_repeat_count (argnum, p, str)
     error (1, 0, "%s: `}' is required in repeat count", str);
   *end = '\0';
 
-  if (!string_to_number (&p->repeat, str +  1))
-    error (1, 0, "%s}: integer required between `{' and `}'",
-	   global_argv[argnum]);
+  if (str+1 == end-1 && *(str+1) == '*')
+    p->repeat = INT_MAX;
+  else
+    if (!string_to_number (&p->repeat, str +  1))
+      error (1, 0, "%s}: integer required between `{' and `}'",
+	     global_argv[argnum]);
 
   *end = '}';
 }
@@ -1200,6 +1276,163 @@ parse_patterns (argc, start, argv)
     }
 }
 
+static unsigned
+get_format_flags (format_ptr)
+     char **format_ptr;
+{
+  unsigned count = 0;
+
+  for (; **format_ptr; (*format_ptr)++)
+    {
+      switch (**format_ptr)
+	{
+	case '-':
+	  break;
+
+	case '+':
+	case ' ':
+	  count++;
+	  break;
+
+	case '#':
+	  count += 2;	/* Allow for 0x prefix preceeding an `x' conversion.  */
+	  break;
+
+	default:
+	  return count;
+	}
+    }
+  return count;
+}
+
+static unsigned
+get_format_width (format_ptr)
+     char **format_ptr;
+{
+  unsigned count = 0;
+  char *start;
+  int ch_save;
+
+  start = *format_ptr;
+  for (; **format_ptr; (*format_ptr)++)
+    if (**format_ptr < '0' || **format_ptr > '9')
+      break;
+
+  ch_save = **format_ptr;
+  **format_ptr = '\0';
+  /* In the case where no minimum field width is explicitly specified,
+     allow for enough octal digits to represent the value of LONG_MAX.  */
+  count = ((*format_ptr == start)
+	   ? bytes_to_octal_digits[sizeof (long)]
+	   : atoi (start));
+  **format_ptr = ch_save;
+  return count;
+}
+
+static unsigned
+get_format_prec (format_ptr)
+     char **format_ptr;
+{
+  unsigned count = 0;
+  char *start;
+  int ch_save;
+  int is_negative;
+
+  if (**format_ptr != '.')
+    return 0;
+  (*format_ptr)++;
+
+  if (**format_ptr == '-' || **format_ptr == '+')
+    {
+      is_negative = (**format_ptr == '-');
+      (*format_ptr)++;
+    }
+  else
+    {
+      is_negative = 0;
+    }
+
+  start = *format_ptr;
+  for (; **format_ptr; (*format_ptr)++)
+    if (**format_ptr < '0' || **format_ptr > '9')
+      break;
+
+  /* ANSI 4.9.6.1 says that if the precision is negative, it's as good as
+     not there. */
+  if (is_negative)
+    start = *format_ptr;
+
+  ch_save = **format_ptr;
+  **format_ptr = '\0';
+  count = (*format_ptr == start) ? 11 : atoi (start);
+  **format_ptr = ch_save;
+
+  return count;
+}
+
+static void
+get_format_conv_type (format_ptr)
+     char **format_ptr;
+{
+  int ch = *((*format_ptr)++);
+
+  switch (ch)
+    {
+    case 'd':
+    case 'i':
+    case 'o':
+    case 'u':
+    case 'x':
+    case 'X':
+      break;
+
+    case 0:
+      error (1, 0, "missing conversion specifier in suffix");
+      break;
+
+    default:
+      if (ISPRINT (ch))
+        error (1, 0, "invalid conversion specifier in suffix: %c", ch);
+      else
+	error (1, 0, "invalid conversion specifier in suffix: \\%.3o", ch);
+    }
+}
+
+static unsigned
+max_out (format)
+     char *format;
+{
+  unsigned out_count = 0;
+  unsigned percents = 0;
+
+  for (; *format; )
+    {
+      int ch = *format++;
+
+      if (ch != '%')
+        out_count++;
+      else
+	{
+	  percents++;
+	  out_count += get_format_flags (&format);
+	  {
+	    int width = get_format_width (&format);
+	    int prec = get_format_prec (&format);
+
+	    out_count += MAX (width, prec);
+	  }
+	  get_format_conv_type (&format);
+	}
+    }
+
+  if (percents == 0)
+    error (1, 0, "missing %% conversion specification in suffix");
+  else if (percents > 1)
+    error (1, 0, "too many %% conversion specifications in suffix");
+
+  return out_count;
+}
+
 static void
 interrupt_handler ()
 {
@@ -1256,7 +1489,7 @@ main (argc, argv)
     signal (SIGTERM, interrupt_handler);
 #endif
 
-  while ((optc = getopt_long (argc, argv, "f:kn:s", longopts, (int *) 0))
+  while ((optc = getopt_long (argc, argv, "f:b:kn:sqz", longopts, (int *) 0))
 	 != EOF)
     switch (optc)
       {
@@ -1267,6 +1500,10 @@ main (argc, argv)
 	prefix = optarg;
 	break;
 
+      case 'b':
+	suffix = optarg;
+	break;
+
       case 'k':
 	remove_files = FALSE;
 	break;
@@ -1277,26 +1514,34 @@ main (argc, argv)
 	break;
 
       case 's':
+      case 'q':
 	suppress_count = TRUE;
 	break;
 
+      case 'z':
+	elide_empty_files = TRUE;
+	break;
+
       default:
-	usage ();
+	usage (1);
       }
 
-  if (flag_version)
+  if (show_version)
     {
-      fprintf (stderr, "%s\n", version_string);
+      printf ("%s\n", version_string);
       exit (0);
     }
 
-  if (flag_help)
-    usage ();
+  if (show_help)
+    usage (0);
 
   if (optind >= argc - 1)
-    usage ();
+    usage (1);
 
-  filename_space = (char *) xmalloc (strlen (prefix) + digits + 2);
+  if (suffix)
+    filename_space = (char *) xmalloc (strlen (prefix) + max_out (suffix) + 2);
+  else
+    filename_space = (char *) xmalloc (strlen (prefix) + digits + 2);
 
   set_input_file (argv[optind++]);
 
@@ -1314,12 +1559,39 @@ main (argc, argv)
 }
 
 static void
-usage ()
+usage (status)
+     int status;
 {
-  fprintf (stderr, "\
-Usage: %s [-sk] [-f prefix] [-n digits] [--prefix=prefix]\n\
-       [--digits=digits] [--quiet] [--silent] [--keep-files]\n\
-       [--help] [--version] file pattern...\n",
-	   program_name);
-  exit (1);
+  if (status != 0)
+    fprintf (stderr, "Try `%s --help' for more information.\n",
+	     program_name);
+  else
+    {
+      printf ("\
+Usage: %s [OPTION]... FILE PATTERN...\n\
+",
+	      program_name);
+      printf ("\
+\n\
+  -b, --suffix=FORMAT        use sprintf FORMAT instead of %%d\n\
+  -f, --prefix=PREFIX        use PREFIX instead of xx\n\
+  -k, --keep-files           do not remove output files on errors\n\
+  -n, --digits=DIGITS        use specified number of digits instead of 2\n\
+  -s, --quiet, --silent      do not print counts of output file sizes\n\
+  -z, --elide-empty-files    remove empty output files\n\
+      --help                 display this help and exit\n\
+      --version              output version information and exit\n\
+\n\
+Read standard input if FILE is -.  Each PATTERN may be:\n\
+\n\
+  INTEGER            copy up to but not including specified line number\n\
+  /REGEXP/[OFFSET]   copy up to but not including a matching line\n\
+  %%REGEXP%%[OFFSET]   skip to, but not including a matching line\n\
+  {INTEGER}          repeat the previous pattern specified number of times\n\
+  {*}                repeat the previous pattern as many times as possible\n\
+\n\
+A line OFFSET is a `+' or `-' (required) followed by a positive integer.\n\
+");
+    }
+  exit (status);
 }
