@@ -1,6 +1,6 @@
 /* cut - remove parts of lines of files
    Copyright (C) 1984 by David M. Ihnat
- 
+
    This program is a total rewrite of the Bell Laboratories Unix(Tm)
    command of the same name, as of System V.  It contains no proprietary
    code, and therefore may be used without violation of any proprietary
@@ -57,6 +57,17 @@
 
    A FILE of `-' means standard input. */
 
+#ifdef HAVE_CONFIG_H
+#if defined (CONFIG_BROKETS)
+/* We use <config.h> instead of "config.h" so that a compilation
+   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
+   (which it would do because it found this file in $srcdir).  */
+#include <config.h>
+#else
+#include "config.h"
+#endif
+#endif
+
 /* Get isblank from GNU libc.  */
 #define _GNU_SOURCE
 
@@ -135,11 +146,15 @@ static unsigned char delim;
 /* Nonzero if we have ever read standard input. */
 static int have_read_stdin;
 
-/* If non-zero, display usage information and exit.  */
-static int flag_help;
+/* If nonzero, this is the index of the first field in a range that goes
+   to end of line. */
+static int eol_range_start;
 
-/* If non-zero, print the version on standard error.  */
-static int flag_version;
+/* If non-zero, display usage information and exit.  */
+static int show_help;
+
+/* If non-zero, print the version on standard output then exit.  */
+static int show_version;
 
 static struct option const longopts[] =
 {
@@ -148,8 +163,8 @@ static struct option const longopts[] =
   {"fields", required_argument, 0, 'f'},
   {"delimiter", required_argument, 0, 'd'},
   {"only-delimited", no_argument, 0, 's'},
-  {"help", no_argument, &flag_help, 1},
-  {"version", no_argument, &flag_version, 1},
+  {"help", no_argument, &show_help, 1},
+  {"version", no_argument, &show_version, 1},
   {0, 0, 0, 0}
 };
 
@@ -188,7 +203,7 @@ main (argc, argv)
 	case 'c':
 	  /* Build the byte list. */
 	  if (operating_mode != undefined_mode)
-	    usage ();
+	    usage (2);
 	  operating_mode = byte_mode;
 	  if (set_fields (optarg) == 0)
 	    error (2, 0, "no fields given");
@@ -197,7 +212,7 @@ main (argc, argv)
 	case 'f':
 	  /* Build the field list. */
 	  if (operating_mode != undefined_mode)
-	    usage ();
+	    usage (2);
 	  operating_mode = field_mode;
 	  if (set_fields (optarg) == 0)
 	    error (2, 0, "no fields given");
@@ -220,24 +235,24 @@ main (argc, argv)
 	  break;
 
 	default:
-	  usage ();
+	  usage (2);
 	}
     }
 
-  if (flag_version)
+  if (show_version)
     {
-      fprintf (stderr, "%s\n", version_string);
+      printf ("%s\n", version_string);
       exit (0);
     }
 
-  if (flag_help)
-    usage ();
+  if (show_help)
+    usage (0);
 
   if (operating_mode == undefined_mode)
-    usage ();
+    usage (2);
 
   if ((delimited_lines_only || delim != '\0') && operating_mode != field_mode)
-    usage ();
+    usage (2);
 
   if (delim == '\0')
     delim = '\t';
@@ -275,8 +290,6 @@ set_fields (fieldstr)
   int dash_found = 0;		/* Nonzero if a '-' is found in this field. */
   int value = 0;		/* If nonzero, a number being accumulated. */
   int fields_selected = 0;	/* Number of fields selected so far. */
-  /* If nonzero, index of first field in a range that goes to end of line. */
-  int eol_range_start = 0;
 
   for (;;)
     {
@@ -540,8 +553,13 @@ cut_fields (stream)
       if (fieldfound)
 	{
 	  /* Something was found. Print it. */
-	  if ((unsigned char) outbufptr[-1] == delim)
-	    --outbufptr;	/* Suppress trailing delimiter. */
+
+	  if ((unsigned char) outbufptr[-1] == delim && eol_range_start == 0)
+	    {
+	      /* Suppress the trailing delimiter unless there is a range
+		 extending to end of line. */
+	      --outbufptr;
+	    }
 
 	  fwrite (outbuf, sizeof (char), outbufptr - outbuf, stdout);
 	  if (c == '\n')
@@ -588,14 +606,39 @@ invalid_list ()
 }
 
 static void
-usage ()
+usage (status)
+     int status;
 {
-  fprintf (stderr, "\
-Usage: %s {-b byte-list,--bytes=byte-list} [-n] [file...] <options> \n\
-       %s {-c character-list,--characters=character-list} <options> [file...]\n\
-       %s {-f field-list,--fields=field-list} [-d delim] [-s] \n\
-       [--delimiter=delim] [--only-delimited] <options> [file...]\n\
-  Options: [--help] [--version]\n",
-	   program_name, program_name, program_name);
-  exit (2);
+  if (status != 0)
+    fprintf (stderr, "Try `%s --help' for more information.\n",
+	     program_name);
+  else
+    {
+      printf ("\
+Usage: %s [OPTION]... [FILE]...\n\
+",
+	      program_name);
+      printf ("\
+\n\
+  -b, --bytes=LIST        output only these bytes\n\
+  -c, --characters=LIST   output only these characters\n\
+  -d, --delimiter=DELIM   use DELIM instead of TAB for field delimiter\n\
+  -f, --fields=LIST       output only these fields\n\
+  -n                      (ignored)\n\
+  -s, --only-delimited    do not print lines not containing delimiters\n\
+      --help              display this help and exit\n\
+      --version           output version information and exit\n\
+\n\
+Use one, and only one of -b, -c or -f.  Each LIST is made up of one\n\
+range, or many ranges separated by commas.  Each range is one of:\n\
+\n\
+  N     N'th byte, character or field, counted from 1\n\
+  N-    from N'th byte, character or field, to end of line\n\
+  N-M   from N'th to M'th (included) byte, character or field\n\
+  -M    from first to M'th (included) byte, character or field\n\
+\n\
+With no FILE, or when FILE is -, read standard input.\n\
+");
+    }
+  exit (status);
 }
