@@ -33,6 +33,17 @@
 
    David MacKenzie <djm@gnu.ai.mit.edu> */
 
+#ifdef HAVE_CONFIG_H
+#if defined (CONFIG_BROKETS)
+/* We use <config.h> instead of "config.h" so that a compilation
+   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
+   (which it would do because it found this file in $srcdir).  */
+#include <config.h>
+#else
+#include "config.h"
+#endif
+#endif
+
 /* Get isblank from GNU libc.  */
 #define _GNU_SOURCE
 
@@ -96,17 +107,17 @@ static int have_read_stdin;
 static int exit_status;
 
 /* If non-zero, display usage information and exit.  */
-static int flag_help;
+static int show_help;
 
-/* If non-zero, print the version on standard error.  */
-static int flag_version;
+/* If non-zero, print the version on standard output then exit.  */
+static int show_version;
 
 static struct option const longopts[] =
 {
   {"tabs", required_argument, NULL, 't'},
   {"initial", no_argument, NULL, 'i'},
-  {"help", no_argument, &flag_help, 1},
-  {"version", no_argument, &flag_version, 1},
+  {"help", no_argument, &show_help, 1},
+  {"version", no_argument, &show_version, 1},
   {NULL, 0, NULL, 0}
 };
 
@@ -134,7 +145,7 @@ main (argc, argv)
 	  break;
 
 	case '?':
-	  usage ();
+	  usage (1);
 	case 'i':
 	  convert_entire_line = 0;
 	  break;
@@ -153,14 +164,14 @@ main (argc, argv)
 	}
     }
 
-  if (flag_version)
+  if (show_version)
     {
-      fprintf (stderr, "%s\n", version_string);
+      printf ("%s\n", version_string);
       exit (0);
     }
 
-  if (flag_help)
-    usage ();
+  if (show_help)
+    usage (0);
 
   add_tabstop (tabval);
 
@@ -241,7 +252,7 @@ validate_tabstops (tabs, entries)
 {
   int prev_tab = 0;
   int i;
-  
+
   for (i = 0; i < entries; i++)
     {
       if (tabs[i] == 0)
@@ -380,11 +391,29 @@ next_file (fp)
 }
 
 static void
-usage ()
+usage (status)
+     int status;
 {
-  fprintf (stderr, "\
-Usage: %s [-tab1[,tab2[,...]]] [-t tab1[,tab2[,...]]] [-i]\n\
-       [--tabs=tab1[,tab2[,...]]] [--initial] [--help] [--version] [file...]\n",
-	   program_name);
-  exit (1);
+  if (status != 0)
+    fprintf (stderr, "Try `%s --help' for more information.\n",
+	     program_name);
+  else
+    {
+      printf ("\
+Usage: %s [OPTION]... [FILE]...\n\
+",
+	      program_name);
+      printf ("\
+\n\
+  -i, --initial       do not convert TABs after non whitespace\n\
+  -t, --tabs=NUMBER   have tabs NUMBER characters apart, not 8\n\
+  -t, --tabs=LIST     use comma separated list of explicit tab positions\n\
+      --help          display this help and exit\n\
+      --version       output version information and exit\n\
+\n\
+Instead of -t NUMBER or -t LIST, -NUMBER or -LIST may be used.  With\n\
+no FILE, or when FILE is -, read standard input.\n\
+");
+    }
+  exit (status);
 }
