@@ -17,6 +17,17 @@
 
 /* Written by David MacKenzie, djm@gnu.ai.mit.edu. */
 
+#ifdef HAVE_CONFIG_H
+#if defined (CONFIG_BROKETS)
+/* We use <config.h> instead of "config.h" so that a compilation
+   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
+   (which it would do because it found this file in $srcdir).  */
+#include <config.h>
+#else
+#include "config.h"
+#endif
+#endif
+
 /* Get isblank from GNU libc.  */
 #define _GNU_SOURCE
 
@@ -46,29 +57,42 @@ static int count_bytes;
 static int have_read_stdin;
 
 /* If non-zero, display usage information and exit.  */
-static int flag_help;
+static int show_help;
 
-/* If non-zero, print the version on standard error.  */
-static int flag_version;
+/* If non-zero, print the version on standard output then exit.  */
+static int show_version;
 
 static struct option const longopts[] =
 {
   {"bytes", no_argument, NULL, 'b'},
   {"spaces", no_argument, NULL, 's'},
   {"width", required_argument, NULL, 'w'},
-  {"help", no_argument, &flag_help, 1},
-  {"version", no_argument, &flag_version, 1},
+  {"help", no_argument, &show_help, 1},
+  {"version", no_argument, &show_version, 1},
   {NULL, 0, NULL, 0}
 };
 
 static void
-usage ()
+usage (status)
+     int status;
 {
-  fprintf (stderr, "\
-Usage: %s [-bs] [-w width] [--bytes] [--spaces] [--width=width]\n\
-       [--help] [--version] [file...]\n",
-	   program_name);
-  exit (1);
+  if (status != 0)
+    fprintf (stderr, "Try `%s --help' for more information.\n",
+	     program_name);
+  else
+    {
+      printf ("\
+Usage: %s [OPTION]... [FILE]...\n\
+",
+	      program_name);
+      printf ("\
+\n\
+  -b, --bytes         count bytes rather than columns\n\
+  -s, --spaces        break at word boundaries\n\
+  -w, --width=WIDTH   use WIDTH columns instead of 80\n\
+");
+    }
+  exit (status);
 }
 
 void
@@ -122,18 +146,18 @@ main (argc, argv)
 	  break;
 
 	default:
-	  usage ();
+	  usage (1);
 	}
     }
 
-  if (flag_version)
+  if (show_version)
     {
-      fprintf (stderr, "%s\n", version_string);
+      printf ("%s\n", version_string);
       exit (0);
     }
 
-  if (flag_help)
-    usage ();
+  if (show_help)
+    usage (0);
 
   if (argc == optind)
     errs |= fold_file ("-", width);
@@ -186,7 +210,7 @@ fold_file (filename, width)
 	  allocated_out += 1024;
 	  line_out = xrealloc (line_out, allocated_out);
 	}
-      
+
       if (c == '\n')
 	{
 	  line_out[offset_out++] = c;
@@ -230,6 +254,14 @@ fold_file (filename, width)
 		  goto rescan;
 		}
 	    }
+	  else
+	    {
+	      if (offset_out == 0)
+		{
+		  line_out[offset_out++] = c;
+		  continue;
+		}
+	    }
 	  line_out[offset_out++] = '\n';
 	  fwrite (line_out, sizeof (char), offset_out, stdout);
 	  column = offset_out = 0;
