@@ -17,6 +17,17 @@
 
    Written by Mike Haertel, mike@gnu.ai.mit.edu. */
 
+#ifdef HAVE_CONFIG_H
+#if defined (CONFIG_BROKETS)
+/* We use <config.h> instead of "config.h" so that a compilation
+   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
+   (which it would do because it found this file in $srcdir).  */
+#include <config.h>
+#else
+#include "config.h"
+#endif
+#endif
+
 /* Get isblank from GNU libc.  */
 #define _GNU_SOURCE
 
@@ -25,6 +36,7 @@
 #include <getopt.h>
 #include "system.h"
 #include "version.h"
+#include "long-options.h"
 
 char *xmalloc ();
 char *xrealloc ();
@@ -93,12 +105,6 @@ static struct outlist *outlist_end;
    tab character. */
 static char tab;
 
-/* If non-zero, display usage information and exit.  */
-static int flag_help;
-
-/* If non-zero, print the version on standard error.  */
-static int flag_version;
-
 /* When using getopt_long_only, no long option can start with
    a character that is a short option. */
 static struct option const longopts[] =
@@ -106,8 +112,6 @@ static struct option const longopts[] =
   {"j", required_argument, NULL, 'j'},
   {"j1", required_argument, NULL, '1'},
   {"j2", required_argument, NULL, '2'},
-  {"help", no_argument, &flag_help, 1},
-  {"version", no_argument, &flag_version, 1},
   {NULL, 0, NULL, 0}
 };
 
@@ -575,6 +579,9 @@ main (argc, argv)
   int optc, prev_optc = 0, nfiles, val;
 
   program_name = argv[0];
+
+  parse_long_options (argc, argv, usage);
+
   nfiles = 0;
   print_pairables = 1;
 
@@ -650,27 +657,18 @@ main (argc, argv)
 	    }
 
 	  if (nfiles > 1)
-	    usage ();
+	    usage (1);
 	  names[nfiles++] = optarg;
 	  break;
 
 	case '?':
-	  usage ();
+	  usage (1);
 	}
       prev_optc = optc;
     }
 
-  if (flag_version)
-    {
-      fprintf (stderr, "%s\n", version_string);
-      exit (0);
-    }
-
-  if (flag_help)
-    usage ();
-  
   if (nfiles != 2)
-    usage ();
+    usage (1);
 
   fp1 = strcmp (names[0], "-") ? fopen (names[0], "r") : stdin;
   if (!fp1)
@@ -691,11 +689,38 @@ main (argc, argv)
 }
 
 static void
-usage ()
+usage (status)
+     int status;
 {
-  fprintf (stderr, "\
-Usage: %s [-a 1|2] [-v 1|2] [-e empty-string] [-o field-list...] [-t char]\n\
-       [-j[1|2] field] [-1 field] [-2 field] file1 file2\n",
-	   program_name);
-  exit (1);
+  if (status != 0)
+    fprintf (stderr, "Try `%s --help' for more information.\n",
+	     program_name);
+  else
+    {
+      printf ("\
+Usage: %s [OPTION]... FILE1 FILE2\n\
+",
+	      program_name);
+      printf ("\
+\n\
+  -a SIDE          print unpairable lines coming from file SIDE\n\
+  -e EMPTY         replace missing input fields with EMPTY\n\
+  -j FIELD         join on this FIELD for both files\n\
+  -[j]SIDE FIELD   join on this FIELD for file SIDE\n\
+  -o FORMAT        obey FORMAT while constructing output line\n\
+  -t CHAR          use CHAR as input and output field separator\n\
+  -v SIDE          like -a SIDE, but suppress joined output lines\n\
+  --help           display this help and exit\n\
+  --version        output version information and exit\n\
+\n\
+When FILE1 or FILE2 is -, not both, read standard input.  SIDE is 1\n\
+for FILE1 or 2 for FILE2.  Unless -t CHAR is given, leading blanks\n\
+separate fields and are ignored, else fields are separated by CHAR.\n\
+Any FIELD is a field number counted from 1.  FORMAT is one or more\n\
+comma or blank separated specifications, each being `SIDE.FIELD'.\n\
+Default FORMAT outputs the join field, the remaining fields from\n\
+FILE1, the remaining fields from FILE2, all separated by CHAR.\n\
+");
+    }
+  exit (status);
 }
