@@ -1,6 +1,6 @@
 /* paste - merge lines of files
    Copyright (C) 1984 by David M. Ihnat
- 
+
    This program is a total rewrite of the Bell Laboratories Unix(Tm)
    command of the same name, as of System V.  It contains no proprietary
    code, and therefore may be used without violation of any proprietary
@@ -16,16 +16,16 @@
    this program accepts full responsibility for any effects or loss;
    in particular, the author is not responsible for any losses,
    explicit or incidental, that may be incurred through use of this program.
- 
+
    I ask that any bugs (and, if possible, fixes) be reported to me when
    possible.  -David Ihnat (312) 784-4544 ignatz@homebru.chi.il.us
 
    The list of valid escape sequences has been expanded over the Unix
    version, to include \b, \f, \r, and \v.
- 
+
    POSIX changes, bug fixes, long-named options, and cleanup
    by David MacKenzie <djm@gnu.ai.mit.edu>.
- 
+
    Options:
    --serial
    -s				Paste one file at a time rather than
@@ -38,6 +38,17 @@
    A FILE of `-' means standard input.
    If no FILEs are given, standard input is used. */
 
+#ifdef HAVE_CONFIG_H
+#if defined (CONFIG_BROKETS)
+/* We use <config.h> instead of "config.h" so that a compilation
+   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
+   (which it would do because it found this file in $srcdir).  */
+#include <config.h>
+#else
+#include "config.h"
+#endif
+#endif
+
 #include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
@@ -79,17 +90,17 @@ static char *delims;
 static char *delim_end;
 
 /* If non-zero, display usage information and exit.  */
-static int flag_help;
+static int show_help;
 
-/* If non-zero, print the version on standard error.  */
-static int flag_version;
+/* If non-zero, print the version on standard output then exit.  */
+static int show_version;
 
 static struct option const longopts[] =
 {
   {"serial", no_argument, 0, 's'},
   {"delimiters", required_argument, 0, 'd'},
-  {"help", no_argument, &flag_help, 1},
-  {"version", no_argument, &flag_version, 1},
+  {"help", no_argument, &show_help, 1},
+  {"version", no_argument, &show_version, 1},
   {0, 0, 0, 0}
 };
 
@@ -128,18 +139,18 @@ main (argc, argv)
 	  break;
 
 	default:
-	  usage ();
+	  usage (1);
 	}
     }
 
-  if (flag_version)
+  if (show_version)
     {
-      fprintf (stderr, "%s\n", version_string);
+      printf ("%s\n", version_string);
       exit (0);
     }
 
-  if (flag_help)
-    usage ();
+  if (show_help)
+    usage (0);
 
   if (optind == argc)
     argv[argc++] = "-";
@@ -333,7 +344,7 @@ paste_parallel (nfiles, fnamptr)
 		      error (0, errno, "%s", fnamptr[i]);
 		      errors = 1;
 		    }
-		      
+
 		  fileptr[i] = CLOSED;
 		  files_open--;
 		}
@@ -470,11 +481,27 @@ paste_serial (nfiles, fnamptr)
 }
 
 static void
-usage ()
+usage (status)
+     int status;
 {
-  fprintf (stderr, "\
-Usage: %s [-s] [-d delim-list] [--serial] [--delimiters=delim-list]\n\
-       [--help] [--version] [file...]\n",
-	   program_name);
-  exit (1);
+  if (status != 0)
+    fprintf (stderr, "Try `%s --help' for more information.\n",
+	     program_name);
+  else
+    {
+      printf ("\
+Usage: %s [OPTION]... [FILE]...\n\
+",
+	      program_name);
+      printf ("\
+\n\
+  -d, --delimiters=LIST   reuse characters from LIST instead of TABs\n\
+  -s, --serial            paste one file at a time instead of in parallel\n\
+      --help              display this help and exit\n\
+      --version           output version information and exit\n\
+\n\
+With no INPUT, or when INPUT is -, read standard input.\n\
+");
+    }
+  exit (status);
 }
