@@ -17,6 +17,17 @@
 
 /* Written by Richard Stallman and David MacKenzie. */
 
+#ifdef HAVE_CONFIG_H
+#if defined (CONFIG_BROKETS)
+/* We use <config.h> instead of "config.h" so that a compilation
+   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
+   (which it would do because it found this file in $srcdir).  */
+#include <config.h>
+#else
+#include "config.h"
+#endif
+#endif
+
 /* Get isblank from GNU libc.  */
 #define _GNU_SOURCE
 
@@ -70,10 +81,10 @@ enum output_mode
 static enum output_mode mode;
 
 /* If non-zero, display usage information and exit.  */
-static int flag_help;
+static int show_help;
 
-/* If non-zero, print the version on standard error.  */
-static int flag_version;
+/* If non-zero, print the version on standard output then exit.  */
+static int show_version;
 
 static struct option const longopts[] =
 {
@@ -83,8 +94,8 @@ static struct option const longopts[] =
   {"skip-fields", required_argument, NULL, 'f'},
   {"skip-chars", required_argument, NULL, 's'},
   {"check-chars", required_argument, NULL, 'w'},
-  {"help", no_argument, &flag_help, 1},
-  {"version", no_argument, &flag_version, 1},
+  {"help", no_argument, &show_help, 1},
+  {"version", no_argument, &show_version, 1},
   {NULL, 0, NULL, 0}
 };
 
@@ -143,24 +154,24 @@ main (argc, argv)
 	case 'u':
 	  mode = output_unique;
 	  break;
-	  
+
 	case 'w':
 	  check_chars = atoi (optarg);
 	  break;
-	  
+
 	default:
-	  usage ();
+	  usage (1);
 	}
     }
 
-  if (flag_version)
+  if (show_version)
     {
-      fprintf (stderr, "%s\n", version_string);
+      printf ("%s\n", version_string);
       exit (0);
     }
 
-  if (flag_help)
-    usage ();
+  if (show_help)
+    usage (0);
 
   if (optind >= 2 && strcmp (argv[optind - 1], "--") != 0)
     {
@@ -177,7 +188,7 @@ main (argc, argv)
     outfile = argv[optind++];
 
   if (optind < argc)
-    usage ();			/* Extra arguments. */
+    usage (1);			/* Extra arguments. */
 
   check_file (infile, outfile);
 
@@ -334,13 +345,35 @@ writeline (line, stream, linecount)
 }
 
 static void
-usage ()
+usage (status)
+     int status;
 {
-  fprintf (stderr, "\
-Usage: %s [-cdu] [-f skip-fields] [-s skip-chars] [-w check-chars]\n\
-       [-#skip-fields] [+#skip-chars] [--count] [--repeated] [--unique]\n\
-       [--skip-fields=skip-fields] [--skip-chars=skip-chars]\n\
-       [--check-chars=check-chars] [--help] [--version] [infile] [outfile]\n",
-	   program_name);
-  exit (1);
+  if (status != 0)
+    fprintf (stderr, "Try `%s --help' for more information.\n",
+	     program_name);
+  else
+    {
+      printf ("\
+Usage: %s [OPTION]... [INPUT [OUTPUT]]\n\
+",
+	      program_name);
+      printf ("\
+\n\
+  -c, --count           prefix lines by the number of occurrences\n\
+  -d, --repeated        only print duplicate lines\n\
+  -f, --skip-fields=N   avoid comparing the N first fields\n\
+  -s, --skip-chars=N    avoid comparing the N first characters\n\
+  -u, --unique          only print unique lines\n\
+  -w, --check-chars=N   compare no more then N characters in lines\n\
+  -N                    same as -f N\n\
+  +N                    same as -s N\n\
+      --help            display this help and exit\n\
+      --version         output version information and exit\n\
+\n\
+A field is a run of whitespace, than non whitespace characters.\n\
+Fields are skipped before chars.  If OUTPUT not specified, writes to\n\
+standard output.  If INPUT not specified, reads from standard input.\n\
+");
+    }
+  exit (status);
 }
