@@ -18,6 +18,17 @@
 /* Written by Paul Rubin, phr@ocf.berkeley.edu
    and David MacKenzie, djm@gnu.ai.mit.edu. */
 
+#ifdef HAVE_CONFIG_H
+#if defined (CONFIG_BROKETS)
+/* We use <config.h> instead of "config.h" so that a compilation
+   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
+   (which it would do because it found this file in $srcdir).  */
+#include <config.h>
+#else
+#include "config.h"
+#endif
+#endif
+
 #include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
@@ -49,10 +60,10 @@ static int have_read_stdin;
 static int exit_status;
 
 /* If non-zero, display usage information and exit.  */
-static int flag_help;
+static int show_help;
 
-/* If non-zero, print the version on standard error.  */
-static int flag_version;
+/* If non-zero, print the version on standard output then exits.  */
+static int show_version;
 
 static struct option const longopts[] =
 {
@@ -60,19 +71,37 @@ static struct option const longopts[] =
   {"chars", no_argument, NULL, 'c'},
   {"lines", no_argument, NULL, 'l'},
   {"words", no_argument, NULL, 'w'},
-  {"help", no_argument, &flag_help, 1},
-  {"version", no_argument, &flag_version, 1},
+  {"help", no_argument, &show_help, 1},
+  {"version", no_argument, &show_version, 1},
   {NULL, 0, NULL, 0}
 };
 
 static void
-usage ()
+usage (status)
+     int status;
 {
-  fprintf (stderr, "\
-Usage: %s [-clw] [--bytes] [--chars] [--lines] [--words]\n\
-       [--help] [--version] [file...]\n",
-	   program_name);
-  exit (1);
+  if (status != 0)
+    fprintf (stderr, "Try `%s --help' for more information.\n",
+	     program_name);
+  else
+    {
+      printf ("\
+Usage: %s [OPTION]... [FILE]...\n\
+",
+	      program_name);
+      printf ("\
+\n\
+  -c, --bytes, --chars   print the byte counts\n\
+  -l, --lines            print the newline counts\n\
+  -w, --words            print the word counts\n\
+      --help             display this help and exit\n\
+      --version          output version information and exit\n\
+\n\
+Print lines, words and bytes in that order.  If none of -clw, select\n\
+them all.  With no FILE, or when FILE is -, read standard input.\n\
+");
+    }
+  exit (status);
 }
 
 void
@@ -107,17 +136,17 @@ main (argc, argv)
 	break;
 
       default:
-	usage ();
+	usage (1);
       }
 
-  if (flag_version)
+  if (show_version)
     {
-      fprintf (stderr, "%s\n", version_string);
+      printf ("%s\n", version_string);
       exit (0);
     }
 
-  if (flag_help)
-    usage ();
+  if (show_help)
+    usage (0);
 
   if (print_lines + print_words + print_chars == 0)
     print_lines = print_words = print_chars = 1;
